﻿using com.citizen.sdk;
using System;
using System.Diagnostics;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Citizen_POS_sample1_CS {
	/// <summary>
	/// An empty page that can be used on its own or navigated to within a Frame.
	/// </summary>
	public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            comboBox.Items.Add("WiFi");
            comboBox.Items.Add("Bluetooth");
            comboBox.SelectedIndex = 0;

        }

        private async void button_Click(object sender, RoutedEventArgs e)
        {
            int type = comboBox.SelectedIndex == 0 ? ESCPOSConst.CMP_PORT_WiFi : ESCPOSConst.CMP_PORT_Bluetooth;
            string address = textBox.Text;

            // Create an instance.
            ESCPOSPrinter printer = new ESCPOSPrinter();

            // Connect printer
            int result = await printer.ConnectAsync(type, address);
            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // Set encoding
                printer.SetEncoding("ISO-8859-1");      // Latin-1
                //printer.SetEncoding("Shift_JIS");     // Japanese 

                // Max page mode area
                Debug.WriteLine("Max. Page area", "( x,y ) : " + printer.GetPageModeArea());

                // Start Transaction ( Batch )
                await printer.TransactionPrintAsync(ESCPOSConst.CMP_TP_TRANSACTION);

                // Print Text
                await printer.PrintTextAsync("Citizen_POS_sample1_CS\n\n",
                        ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                        ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                await printer.PrintTextAsync("- Sample Print 1 -\n",
                        ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT,
                        ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_2HEIGHT);
                await printer.PrintTextAsync("123456789012345678901234567890123456789012345678901234567890\n",
                        ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT,
                        ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                // Print QRcode
                await printer.PrintQRCodeAsync("http://www.citizen-systems.co.jp/", 6,
                        ESCPOSConst.CMP_QRCODE_EC_LEVEL_L,
                        ESCPOSConst.CMP_ALIGNMENT_RIGHT);
                // Partial Cut with Pre-Feed
                await printer.CutPaperAsync(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // End Transaction ( Batch )
                result = await printer.TransactionPrintAsync(ESCPOSConst.CMP_TP_NORMAL);

                // Disconnect
                await printer.DisconnectAsync();

                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    // Print process Error
                    MessageDialog msgbox = new MessageDialog("Transaction Error : " + result.ToString(), "Citizen_POS_sample1");
                    await msgbox.ShowAsync();
                }
            }
            else
            {
                // Connect Error
                MessageDialog msgbox = new MessageDialog("Connect Error : " + result.ToString(), "Citizen_POS_sample1");
                await msgbox.ShowAsync();
            }

        }

		private async void button1_Click(object sender, RoutedEventArgs e)
        {
            int type = comboBox.SelectedIndex == 0 ? ESCPOSConst.CMP_PORT_WiFi : ESCPOSConst.CMP_PORT_Bluetooth;
            string address = textBox.Text;

            // Create an instance.
            ESCPOSPrinter printer = new ESCPOSPrinter();

            // Connect printer
            int result = await printer.ConnectAsync(type, address);
            if (ESCPOSConst.CMP_SUCCESS == result)
            {
                // Set encoding
                printer.SetEncoding("ISO-8859-1");      // Latin-1
                //printer.SetEncoding("Shift_JIS");     // Japanese 

                // Start Transaction ( Batch )
                await printer.TransactionPrintAsync(ESCPOSConst.CMP_TP_TRANSACTION);

                // Print Normal
                await printer.PrintTextAsync("Citizen_POS_sample1_CS\n\n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);
                await printer.PrintNormalAsync("\u001b|2vC\u001b|cA- Sample Print 2 -\n");
                await printer.PrintTextAsync("1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_1WIDTH | ESCPOSConst.CMP_TXT_1HEIGHT);

                // Start Page mode
                await printer.PageModePrintAsync(ESCPOSConst.CMP_PM_PAGE_MODE);

                // Position set
                printer.SetPageModeVerticalPosition(0);
                printer.SetPageModeHorizontalPosition(0);

                // Direction set
                printer.SetPageModePrintDirection(ESCPOSConst.CMP_PD_TOP_TO_BOTTOM);

                // Print data output
                printer.SetPageModePrintArea("500,0,76,800");
                await printer.PrintNormalAsync("\u001b|4C- Receipt -\n");
                printer.SetPageModePrintArea("260,0,120,800");
                await printer.PrintTextAsync("   $ 299.99-  \n", ESCPOSConst.CMP_ALIGNMENT_CENTER, ESCPOSConst.CMP_FNT_UNDERLINE | ESCPOSConst.CMP_FNT_BOLD, ESCPOSConst.CMP_TXT_4WIDTH | ESCPOSConst.CMP_TXT_4HEIGHT);
                printer.SetPageModePrintArea("88,0,88,560");
                await printer.PrintTextAsync("CITIZEN SYSTEMS\n", ESCPOSConst.CMP_ALIGNMENT_RIGHT, ESCPOSConst.CMP_FNT_DEFAULT, ESCPOSConst.CMP_TXT_2WIDTH | ESCPOSConst.CMP_TXT_3HEIGHT);
                printer.SetPageModePrintArea("0,0,88,480");
                await printer.PrintBarCodeAsync("123456789012", ESCPOSConst.CMP_BCS_UPCA, 64, 4, ESCPOSConst.CMP_ALIGNMENT_LEFT, ESCPOSConst.CMP_HRI_TEXT_BELOW);
                printer.SetPageModePrintArea("0,600,192,192");
                await printer.PrintQRCodeAsync("http://www.citizen-systems.co.jp/", 5, ESCPOSConst.CMP_QRCODE_EC_LEVEL_L, ESCPOSConst.CMP_ALIGNMENT_LEFT);

                // End Page mode
                await printer.PageModePrintAsync(ESCPOSConst.CMP_PM_NORMAL);

                // Partial Cut with Pre-Feed
                await printer.CutPaperAsync(ESCPOSConst.CMP_CUT_PARTIAL_PREFEED);

                // End Transaction ( Batch )
                result = await printer.TransactionPrintAsync(ESCPOSConst.CMP_TP_NORMAL);

                // Disconnect
                await printer.DisconnectAsync();

                if (ESCPOSConst.CMP_SUCCESS != result)
                {
                    // Print process Error
                    MessageDialog msgbox = new MessageDialog("Transaction Error : " + result.ToString(), "Citizen_POS_sample2");
                    await msgbox.ShowAsync();
                }
            }
            else
            {
                // Connect Error
                MessageDialog msgbox = new MessageDialog("Connect Error : " + result.ToString(), "Citizen_POS_sample2");
                await msgbox.ShowAsync();
            }
        }
		//-------------- Line Display Sample --------------
		private async void buttonD_Click(Object sender,RoutedEventArgs e) {
			//Disapble this button while runnning this program
			buttonD.IsEnabled=false;

			//Type of the interface to connect. It depends on setting of the combobox.
			int type = comboBox.SelectedIndex == 0 ? ESCPOSConst.CMP_PORT_WiFi : ESCPOSConst.CMP_PORT_Bluetooth;

			//Address of the interfecse to connect.
			string address = textBox.Text;

			//Create an instance
			LineDisplay display=new LineDisplay();

			int result=await display.ConnectAsync(type,address);
			if(result==LineDisplayConst.CDP_SUCCESS){	//Connected successfully

				display.SetEncoding(437);
				//Get current date and time
				string currentTime=DateTime.Now.ToString(new System.Globalization.CultureInfo("en-US"));
				string[] dt=currentTime.Split(' ');
				string time=dt[1]/*HH:MM:SS*/+" "+dt[2]/*AM or PM*/;
				//display date (M/D/Y) in 1st row
				await display.SetCursorPositionAsync(1,1);
				await display.DisplayTextAsync(dt[0],false);
				//display time in 2nd row
				await display.SetCursorPositionAsync(20-time.Length+1,2);
				await display.DisplayTextAsync(time,false);

				//disconnect display
				await display.DisconnectAsync();
			} else {	//Failed to connect.
				//Show error message
				MessageDialog msgbox = new MessageDialog("Connect Error : " + result.ToString(), "Citizen Display sample");
				await msgbox.ShowAsync();
			}

			//Remove LineDisyplay object from this process
			display=null;

			//Enables button
			buttonD.IsEnabled=true;
		}

		//-------------- Barcode Scanner Sample --------------
		private Scanner scanner=null;	//Scanner object

		private async void buttonS_Click(Object sender,RoutedEventArgs e) {
			//Disapbles this button while runnning this procedure
			buttonS.IsEnabled=false;

			if(scanner!=null) { //Scanner is working
				//Disconnect
				await scanner.DisconnectAsync();

				//Unregister event procedures
				scanner.DataEvent -= DataEvent;
				scanner.StatusUpdateEvent -= StatusUpdateEvent;

				//Remove Scanner object from this process
				scanner=null;

				//Modify change text for scanner button.
				buttonS.Content="Start Scanner";
	   		} else {	//Scanner is not working.
				//Type of the interface to connect. It depends on setting of the combobox.
				int Type = comboBox.SelectedIndex == 0 ? ESCPOSConst.CMP_PORT_WiFi : ESCPOSConst.CMP_PORT_Bluetooth;
				if(Type==ScannerConst.CSC_PORT_WiFi) {
					//WiFi : TCP port is required

					//Create and show sub menu to choose the setting of scanner's interface 
					PopupMenu subMenu=new PopupMenu();
					//menu item 1: HID
					subMenu.Commands.Add(new UICommand("HID",async (comman) => {
						await ConnectScanner(Type,9210);
		   			}));
					//menu item 2: VCOM
					subMenu.Commands.Add(new UICommand("VCOM",async (command) => {
						await ConnectScanner(Type,9201);
					}));
					//Show sub menu on the screen
					await subMenu.ShowForSelectionAsync(GetElementRect((FrameworkElement)sender));
				} else {
					//Bluetooth : port value is ignored

					await ConnectScanner(Type,0);
				}
			}
			//Enables button
			buttonS.IsEnabled=true;
		}
		private static Rect GetElementRect(FrameworkElement element){
		/* Called when menu is shown */
			Windows.UI.Xaml.Media.GeneralTransform buttonTransform = element.TransformToVisual(null);
			Point point = buttonTransform.TransformPoint(new Point());
			return new Rect(point, new Size(element.ActualWidth, element.ActualHeight));
		}

		private async Task ConnectScanner(int type/*Port type*/,int port/*TCP port for ethernet*/) {
			//Initialize Encoding class
			System.Text.Encoding.RegisterProvider(System.Text.CodePagesEncodingProvider.Instance);

			string address = textBox.Text;	//Printer's address
			
			//Create an instance
			scanner=new Scanner();

			//Register event procedures
			scanner.DataEvent += DataEvent;
			scanner.StatusUpdateEvent += StatusUpdateEvent;

			//Connect to the interface.
			int result=await scanner.ConnectAsync(type,address,port);
			if(result!=ScannerConst.CSC_SUCCESS) {	//Failed to connet
				//Connect Error -> show error message
				MessageDialog msgbox = new MessageDialog("Connect Error : " + result.ToString(), "Citizen Scanner sample");
				await msgbox.ShowAsync();

				//Unregister event procedures
				scanner.DataEvent -= DataEvent;
				scanner.StatusUpdateEvent -= StatusUpdateEvent;

				//Remove Scanner object from this process
				scanner=null;

			}else{	//Succeeded to connect
				//Change scanner buttan text
				buttonS.Content="Stop Scanner";
			}
		}

		private async void DataEvent(byte[] data) {
		//Data Event Procedure
			//Cover scan data to text. Japanese text is often used with QRcode.
			string DataText=System.Text.Encoding.GetEncoding(932).GetString(data);
			//show scanned data with current time in a MessageDialog
			await MsgBoxUIAsync(DataText,withTime("DataEvent"));
		}
		private async void StatusUpdateEvent(int status) {
		//Status Update Event Procedure
			//show status with current time in a MessageDialog 
			await MsgBoxUIAsync(status.ToString(),withTime("StatusUpdateEvent"));
		}
		private string withTime(string message) {
		//Appends current time to "message".
			DateTime dt=DateTime.Now;
			return message+string.Format(" ({0}:{1:00}:{2:00})",dt.Hour,dt.Minute,dt.Second);
		}
		private async Task MsgBoxUIAsync(string message,string title) {
		/* MessageDialog cannot be shown from any thread except for the UI thread. 
		   The following method can create a thread which can show MessageDialog. */
			await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal,async ()=>{
				MessageDialog msgbox=new MessageDialog(message,title);
				await msgbox.ShowAsync();
			});
		}
		
	}
}
